# -*- coding: utf-8 -*-

from collections import defaultdict

from kinder.core.dict.models import UnitKind
from kinder.core.declaration.models import Declaration


def get_queue_cache(mo_unit):

    fields = (
        "status__code",
        "date",
        "desired_date",
        "children__health_need__group_type__code",
        "children__health_need_id",
        "children__date_of_birth",
        "best_privilege",
        "best_privilege__type_id",
        "consent_dev_group",
        "children_id",
        "work_type__code",
        "consent_care_group",
        "consent_short_time_group"
    )

    declarations = Declaration.objects.filter(
        mo=mo_unit
    ).prefetch_related(
        "direct_set",
        "declarationunit_set"
    ).select_related(*fields)

    mo_cache = defaultdict(list)
    for declaration in declarations:
        declarations_units = [
            record
            for record in list(declaration.declarationunit_set.all())
            if record.unit.kind_id == UnitKind.DOU
        ]
        if not declarations_units:
            continue
        most_priority_unit = min(
            [
                record
                for record in list(declaration.declarationunit_set.all())
                if record.unit.kind_id == UnitKind.DOU
            ],
            key=lambda rec: rec.ord
        )

        record = {}
        for field in fields:
            value = declaration
            path = field.split("__")
            for el in path:
                value = getattr(value, el)
                if value is None:
                    break
            record[field] = value

        record["directs"] = []
        for direct in declaration.direct_set.all():
            record["directs"].append({
                "status__code": direct.status.code,
                "status__group__unit_id": direct.group.unit_id
            })

        mo_cache[most_priority_unit].append(declaration)

    return mo_cache
